Package: EGAnet
Title: Exploratory Graph Analysis – a Framework for Estimating the
        Number of Dimensions in Multivariate Data using Network
        Psychometrics
Version: 2.4.1
Date: 2026-04-13
Authors@R: c(person("Hudson", "Golino", email = "hfg9s@virginia.edu", role = c("aut", "cre"), comment = c(ORCID = "0000-0002-1601-1447")),
	     person("Alexander", "Christensen", email = "alexpaulchristensen@gmail.com", role = "aut", comment = c(ORCID = "0000-0002-9798-7037")),
	     person("Robert", "Moulder", email = "rgm4fd@virginia.edu", role = "ctb", comment = c(ORCID = "0000-0001-7504-9560")),
	     person("Luis", "E. Garrido", email = "garrido.luiseduardo@gmail.com", role = "ctb", comment = c(ORCID = "0000-0001-8932-6063")),
	     person("Laura", "Jamison", email = "lj5yn@virginia.edu", role = "ctb", comment = c(ORCID = "0000-0002-4656-8684")),
	     person("Dingjing", "Shi", email = "dshi@ou.edu", role = "ctb", comment = c(ORCID = "0000-0002-5652-3818")),
	     person("John", "Fox", role = c("cph", "ctb"), comment = "Original implementation of polyserial correlations in auto_correlate.R"),
    person("Yves", "Rosseel", role = c("cph", "ctb"), comment = "Original implementation of rmsea_ci in network_fit.R"),
    person("Alexander", "Robitzsch", role = c("cph", "ctb"), comment = "C++ implementation of Drezner-Wesolowsky bivariate normal CDF in polychoric_matrix.c"),
    person("David", "Blackman", role = "ctb", comment = "Original xoshiro.c implementation"),
    person("Sebastiano", "Vigna", role = "ctb", comment = "Original xoshiro.c implementation"),
    person("John", "Burkardt", role = c("cph", "ctb"), comment = "Original ziggurat.c implementation"),
    person("Bill", "Venables", role = c("cph", "ctb"), comment = "Original mvrnorm implementation"),
    person("Brian", "Ripley", role = c("cph", "ctb"), comment = "Original mvrnorm implementation"),
    person("Pascal", "Deboeck", role = c("cph", "ctb"), comment = "Original glla implementation"),
    person("Steve", "Boker", role = c("cph", "ctb"), comment = "Original glla implementation"))
Maintainer: Hudson Golino <hfg9s@virginia.edu>
Description: Implements the Exploratory Graph Analysis (EGA) framework for dimensionality
             and psychometric assessment. EGA estimates the number of dimensions in
	     	 psychological data using network estimation methods and community detection
             algorithms. A bootstrap method is provided to assess the stability of dimensions
	     	 and items. Fit is evaluated using the Entropy Fit family of indices. Unique 
             Variable Analysis evaluates the extent to which items are locally dependent (or
             redundant). Network loadings provide similar information to factor loadings and
	     	 can be used to compute network scores. A bootstrap and permutation approach are
             available to assess configural and metric invariance. Hierarchical structures
             can be detected using Hierarchical EGA. Time series and intensive longitudinal 
	     	 data can be analyzed using Dynamic EGA, supporting individual, group, and 
             population level assessments.
Depends: R (>= 3.5.0)
License: AGPL (>= 3.0)
Encoding: UTF-8
LazyData: true
Imports: clue, dendextend, future, future.apply, GGally, ggplot2,
        ggpubr, glasso, glassoFast, GPArotation, igraph (>= 1.3.0),
        lavaan, Matrix, methods, network, progressr, qgraph, semPlot,
        sna, stats
Suggests: fitdistrplus, gridExtra, pbapply, progress, psych, pwr,
        RColorBrewer
Copyright: See inst/COPYRIGHTS for details
URL: https://r-ega.net
BugReports: https://github.com/hfgolino/EGAnet/issues
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-04-13 15:03:55 UTC; alextops
Author: Hudson Golino [aut, cre] (ORCID:
    <https://orcid.org/0000-0002-1601-1447>),
  Alexander Christensen [aut] (ORCID:
    <https://orcid.org/0000-0002-9798-7037>),
  Robert Moulder [ctb] (ORCID: <https://orcid.org/0000-0001-7504-9560>),
  Luis E. Garrido [ctb] (ORCID: <https://orcid.org/0000-0001-8932-6063>),
  Laura Jamison [ctb] (ORCID: <https://orcid.org/0000-0002-4656-8684>),
  Dingjing Shi [ctb] (ORCID: <https://orcid.org/0000-0002-5652-3818>),
  John Fox [cph, ctb] (Original implementation of polyserial correlations
    in auto_correlate.R),
  Yves Rosseel [cph, ctb] (Original implementation of rmsea_ci in
    network_fit.R),
  Alexander Robitzsch [cph, ctb] (C++ implementation of
    Drezner-Wesolowsky bivariate normal CDF in polychoric_matrix.c),
  David Blackman [ctb] (Original xoshiro.c implementation),
  Sebastiano Vigna [ctb] (Original xoshiro.c implementation),
  John Burkardt [cph, ctb] (Original ziggurat.c implementation),
  Bill Venables [cph, ctb] (Original mvrnorm implementation),
  Brian Ripley [cph, ctb] (Original mvrnorm implementation),
  Pascal Deboeck [cph, ctb] (Original glla implementation),
  Steve Boker [cph, ctb] (Original glla implementation)
Repository: CRAN
Date/Publication: 2026-04-13 16:10:02 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-04-13 19:27:19 UTC; unix
Archs: EGAnet.so.dSYM
