# Generated by roxygen2: do not edit by hand

S3method(plot,EGA)
S3method(plot,EGA.estimate)
S3method(plot,EGA.fit)
S3method(plot,boot.ergoInfo)
S3method(plot,bootEGA)
S3method(plot,dynEGA)
S3method(plot,dynEGA.Group)
S3method(plot,dynEGA.Individual)
S3method(plot,dynEGA.Population)
S3method(plot,hierEGA)
S3method(plot,infoCluster)
S3method(plot,invariance)
S3method(plot,itemStability)
S3method(plot,network.compare)
S3method(plot,riEGA)
S3method(plot,tefi.compare)
S3method(print,EGA)
S3method(print,EGA.community)
S3method(print,EGA.consensus)
S3method(print,EGA.estimate)
S3method(print,EGA.fit)
S3method(print,EGA.network)
S3method(print,EII)
S3method(print,UVA)
S3method(print,boot.ergoInfo)
S3method(print,bootEGA)
S3method(print,dimensionStability)
S3method(print,dynEGA)
S3method(print,dynEGA.Group)
S3method(print,dynEGA.Individual)
S3method(print,dynEGA.Population)
S3method(print,dynamic.network.compare)
S3method(print,hierEGA)
S3method(print,infoCluster)
S3method(print,invariance)
S3method(print,itemDiagnostics)
S3method(print,itemStability)
S3method(print,jsd.ergodicity)
S3method(print,net.loads)
S3method(print,network.compare)
S3method(print,predictability)
S3method(print,riEGA)
S3method(print,tefi.compare)
S3method(summary,EGA)
S3method(summary,EGA.community)
S3method(summary,EGA.consensus)
S3method(summary,EGA.estimate)
S3method(summary,EGA.fit)
S3method(summary,EGA.network)
S3method(summary,EII)
S3method(summary,UVA)
S3method(summary,boot.ergoInfo)
S3method(summary,bootEGA)
S3method(summary,dimensionStability)
S3method(summary,dynEGA)
S3method(summary,dynEGA.Group)
S3method(summary,dynEGA.Individual)
S3method(summary,dynEGA.Population)
S3method(summary,dynamic.network.compare)
S3method(summary,hierEGA)
S3method(summary,infoCluster)
S3method(summary,invariance)
S3method(summary,itemDiagnostics)
S3method(summary,itemStability)
S3method(summary,jsd.ergodicity)
S3method(summary,net.loads)
S3method(summary,network.compare)
S3method(summary,predictability)
S3method(summary,riEGA)
S3method(summary,tefi.compare)
export(CFA)
export(EBICglasso.qgraph)
export(EGA)
export(EGA.estimate)
export(EGA.fit)
export(Embed)
export(LCT)
export(TMFG)
export(UVA)
export(auto.correlate)
export(boot.ergoInfo)
export(bootEGA)
export(color_palette_EGA)
export(community.compare)
export(community.consensus)
export(community.detection)
export(community.homogenize)
export(community.unidimensional)
export(compare.EGA.plots)
export(convert2igraph)
export(convert2tidygraph)
export(cosine)
export(dimensionStability)
export(dynEGA)
export(dynEGA.ind.pop)
export(dynamic.network.compare)
export(entropyFit)
export(ergoInfo)
export(genTEFI)
export(glla)
export(hierEGA)
export(igraph2matrix)
export(infoCluster)
export(information)
export(invariance)
export(itemDiagnostics)
export(itemStability)
export(jsd)
export(known.graph)
export(modularity)
export(net.loads)
export(net.scores)
export(network.compare)
export(network.confusion)
export(network.estimation)
export(network.fit)
export(network.predictability)
export(plot_clusters)
export(polychoric.matrix)
export(riEGA)
export(sF)
export(simDFM)
export(simEGM)
export(tefi)
export(tefi.compare)
export(totalCor)
export(totalCorMat)
export(vn.entropy)
export(wto)
importFrom(graphics,par)
importFrom(graphics,text)
importFrom(methods,formalArgs)
importFrom(methods,is)
importFrom(stats,KalmanSmooth)
importFrom(stats,StructTS)
importFrom(stats,as.dendrogram)
importFrom(stats,as.dist)
importFrom(stats,as.hclust)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,cov)
importFrom(stats,cov2cor)
importFrom(stats,cutree)
importFrom(stats,dist)
importFrom(stats,dnorm)
importFrom(stats,hclust)
importFrom(stats,ks.test)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,na.omit)
importFrom(stats,nlm)
importFrom(stats,nlminb)
importFrom(stats,optimHess)
importFrom(stats,optimize)
importFrom(stats,p.adjust)
importFrom(stats,pchisq)
importFrom(stats,pgamma)
importFrom(stats,pnorm)
importFrom(stats,qchisq)
importFrom(stats,qf)
importFrom(stats,qgamma)
importFrom(stats,qnorm)
importFrom(stats,qt)
importFrom(stats,quantile)
importFrom(stats,rnorm)
importFrom(stats,runif)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,t.test)
importFrom(stats,uniroot)
importFrom(stats,var)
importFrom(stats,wilcox.test)
importFrom(utils,browseURL)
importFrom(utils,capture.output)
importFrom(utils,combn)
importFrom(utils,data)
importFrom(utils,globalVariables)
importFrom(utils,object.size)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)
useDynLib(EGAnet, .registration = TRUE)
