#' Prime Numbers through 100,000
#'
#' Numeric vector of primes generated from the primes package. Used in
#' the function \code{[EGAnet]{ergoInfo}}. Not for general use
#'
#' @name prime.num
#'
#' @docType data
#'
#' @usage data(prime.num)
#'
#' @format A 1185x24 response matrix
#'
#' @keywords datasets
#'
#' @examples
#' data("prime.num")
#'
NULL
#----
