#' @title \code{\link[EGAnet]{EGA}} Network of \code{\link[EGAnet]{wmt2}}Data
#'
#' @description \code{\link[EGAnet]{EGA}} results from \code{ega.wmt <- EGA(wmt2[,7:24])}
#' for the Wiener Matrizen-Test (WMT-2)
#' 
#' @name ega.wmt
#'
#' @docType data
#'
#' @usage data(ega.wmt)
#'
#' @format A list with 8 objects (see \strong{Value} in \code{\link[EGAnet]{EGA}})
#'
#' @keywords datasets
#'
#' @examples
#' data("ega.wmt")
NULL