#' Loadings Comparison Test Deep Learning Neural Network Weights
#'
#' A list of weights from four different neural network models:
#' random vs. non-random model (\code{r_nr_weights}),
#' low correlation factor vs. network model (\code{lf_n_weights}),
#' high correlation with variables less than or equal to factors vs. network model (\code{hlf_n_weights}), and
#' high correlation with variables greater than factors vs. network model (\code{hgf_n_weights})
#'
#' @name dnn.weights
#'
#' @docType data
#'
#' @usage data(dnn.weights)
#'
#' @format A list of with a length of 4
#'
#' @keywords datasets
#'
#' @examples
#' data("dnn.weights")
#'
# Updated 17.07.2020
NULL
#----
