#' @title Depression Data
#'
#' @description A response matrix (\emph{n} = 574) of the Beck Depression Inventory, 
#' Beck Anxiety Inventory, and the Athens Insomnia Scale.
#'
#' @name depression
#'
#' @docType data
#'
#' @usage data(depression)
#'
#' @format A 574x78 response matrix
#'
#' @keywords datasets
#'
#' @examples
#' data("depression")
#'
NULL
