#' WMT-2 Data
#'
#' A response matrix (n = 1185) of the Wiener Matrizen-Test 2 (WMT-2).
#'
#' @name wmt2
#'
#' @docType data
#'
#' @usage data(wmt2)
#'
#' @format A 1185x24 response matrix
#'
#' @keywords datasets
#'
#' @examples
#' data("wmt2")
#'
NULL
#----
