#' @title \code{\link[EGAnet]{bootEGA}} Results of \code{\link[EGAnet]{wmt2}}Data
#'
#' @description \code{\link[EGAnet]{bootEGA}} results from \code{boot.wmt <- bootEGA(wmt2[,7:24], seed = 1234)}
#'
#' @name boot.wmt
#'
#' @docType data
#'
#' @usage data(boot.wmt)
#'
#' @format A list with 12 objects (see \strong{Value} in \code{\link[EGAnet]{bootEGA}})
#'
#' @keywords datasets
#'
#' @examples
#' data("boot.wmt")
NULL