#' Intelligence Data
#'
#' A response matrix (n = 1152) of the International Cognitive Ability Resource (ICAR)
#' intelligence battery developed by Condon and Revelle (2016).
#'
#' @name intelligenceBattery
#'
#' @docType data
#'
#' @usage data(intelligenceBattery)
#'
#' @format A 1185x125 response matrix
#'
#' @keywords datasets
#'
#' @examples
#' data("intelligenceBattery")
#'
NULL
#----
