#' Optimism Data
#'
#' A response matrix (n = 282) containing responses to 10 items of the Revised Life
#' Orientation Test (LOT-R), developed by Scheier, Carver, & Bridges (1994).
#'
#' @name optimism
#'
#' @docType data
#'
#' @usage data(optimism)
#'
#' @format A 282x10 response matrix
#'
#' @keywords datasets
#'
#' @references
#' Scheier, M. F., Carver, C. S., & Bridges, M. W. (1994).
#' Distinguishing optimism from neuroticism (and trait anxiety, self-mastery, and self-esteem): a reevaluation of the Life Orientation Test.
#' \emph{Journal of Personality and Social Psychology}, \emph{67}, 1063-1078.
#'
#' @examples
#' data("optimism")
#'
NULL
#----
