% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/depression.R
\docType{data}
\name{depression}
\alias{depression}
\title{Depression Data}
\format{
A 574x78 response matrix
}
\usage{
data(depression)
}
\description{
A response matrix (\emph{n} = 574) of the Beck Depression Inventory, 
Beck Anxiety Inventory, and the Athens Insomnia Scale.
}
\examples{
data("depression")

}
\keyword{datasets}
